"use strict";
require('es6-shim');
/**
 * Fetcher que para realizar requerimientos utiliza una función dada que debe
 * implementar la API fetch (https://developer.mozilla.org/en/docs/Web/API/Fetch_API).
 *
 */
var FetcherBasico = (function () {
    function FetcherBasico(fetchFn) {
        this.fetchFn = fetchFn;
    }
    /**
     * Utiliza la función fetch con la que está configurado el fetcher para obtener y
     * deserializar el JSON de un objeto que se establecerá en la propiedad "datos" de
     * la respuesta.
     * La promesa devuelta se rechazará cuando la función suyacente rechace su promesa o
     * cuando la cumpla con una respuesta con la propiedad "ok" en false-like.
     */
    FetcherBasico.prototype.fetchJson = function (url, init) {
        var _this = this;
        var fetch = this.fetchFn; // que this sea el objeto global de contexto dentro de fetchFn
        var respuesta;
        return fetch(url, init)
            .then(function (resp) {
            respuesta = resp;
            return resp.json();
        })
            .then(function (contenido) {
            if (respuesta.ok) {
                respuesta.datos = contenido;
                return respuesta;
            }
            return Promise.reject(contenido);
        })
            .catch(function (error) { return Promise.reject(_this.normalizarErrores(error, respuesta)); });
    };
    /**
     * Utiliza la función fetch con la que está configurado el fetcher para obtener un
     * texto que se establecerá en la propiedad "datos" de la respuesta.
     * La promesa devuelta se rechazará cuando la función suyacente rechace su promesa o
     * cuando la cumpla con una respuesta con la propiedad "ok" en false-like.
     */
    FetcherBasico.prototype.fetchText = function (url, init) {
        var _this = this;
        var fetch = this.fetchFn; // que this sea el objeto global de contexto dentro de fetchFn
        var respuesta;
        return fetch(url, init)
            .then(function (resp) {
            respuesta = resp;
            return resp.text();
        })
            .then(function (contenido) {
            if (respuesta.ok) {
                respuesta.datos = contenido;
                return respuesta;
            }
            return Promise.reject(contenido);
        })
            .catch(function (error) { return Promise.reject(_this.normalizarErrores(error, respuesta)); });
    };
    /**
     * Utiliza la función fetch con la que está configurado el fetcher para obtener una
     * respuesta.
     * La promesa devuelta se rechazará cuando la función suyacente rechace su promesa o
     * cuando la cumpla con una respuesta con la propiedad "ok" en false-like.
     */
    FetcherBasico.prototype.fetchResponseOk = function (url, init) {
        var _this = this;
        var fetch = this.fetchFn;
        var respuestaOriginal;
        return fetch(url, init)
            .then(function (resp) {
            respuestaOriginal = resp;
            if (resp.ok) {
                return resp;
            }
            return resp
                .json()
                .then(function (errores) { return Promise.reject(errores); }, function () { return Promise.reject(null); }); // No rechazo con los errores de lectura del JSON (mal formado o inexistente) de la respuesta
        })
            .catch(function (errores) { return Promise.reject(_this.normalizarErrores(errores, respuestaOriginal)); });
    };
    FetcherBasico.prototype.normalizarErrores = function (errores, resp) {
        if (!errores || errores.length === 0) {
            return [normalizarError(null, resp)];
        }
        if (!Array.isArray(errores)) {
            errores = [errores];
        }
        return errores.map(function (error) { return normalizarError(error, resp); });
    };
    return FetcherBasico;
}());
exports.FetcherBasico = FetcherBasico;
var codigoErrorGenerico = 'Error';
var mensajeErrorGenerico = 'Error';
function normalizarError(error, resp) {
    var codigoError = error && error.codigoError && error.codigoError + '' ||
        ((resp && resp.status) ? resp.status + '' : codigoErrorGenerico);
    var mensaje = error && (error.mensaje && error.mensaje + '' ||
        typeof error == 'string' && error ||
        error.message && error.message + '') ||
        mensajeErrorGenerico;
    var referencias = error && Array.isArray(error.referencias) ? error.referencias : [];
    return error && error.codigo === codigoError && error.mensaje === mensaje && error.referencias === referencias ?
        error : {
        codigoError: codigoError,
        mensaje: mensaje,
        referencias: referencias
    };
}
